/*
 * @Author: fgg-zero 
 * @Date: 2018-05-30 09:47:42 
 * @Last Modified by: fgg-zero
 * @Last Modified time: 2018-06-25 17:09:43
 */

const Storage = require('./storage')
const Session = require('./session')
const Message = require('./message')
const Prompts = require('./prompts')
const Dialog = require('./conversation/dialog')
const Recognizer = require('./recognizer')

class ChatBot {
  constructor(connector, dialog, settings = {}) {
    this.sessionStorage = {}
    this.storageCtx = {}
    this.settings = settings
    this.conversation = []

    if (dialog) {
      let rootDialog
      if (dialog instanceof Dialog) {
        dialog.id = '/'
        rootDialog = dialog
      } else {
        rootDialog = new Dialog('/', dialog)
      }
      this.rootDialog = rootDialog
      // this.conversation.push(rootDialog)
    }
    if (connector) {
      this.connect(connector);
    }
  }


  connect(connector) {
    const _this = this
    this.conn = connector
    connector.onEvent(function (event, session) { return _this.onReceiveMessage(event, session) })
  }

  /**
   * TODO:: more type of event
   * 
   * @param {*} event 
   * @param {*} cb 
   */
  onReceiveMessage(event, session) {
    let _this = this
    let message = event.content
    // _this.createSession(message, function (err) {
    //   if (err) {
    //     throw new Error('[bot error]: create session error:', err)
    //   }
    _this.startConversation(session)
    // })
  }

  createSession(message, cb) {
    const cid = this.conn.cid
    let session = this.sessionStorage[cid]
    if (!session) {
      this.sessionStorage[cid] = new Session()
    }

    this.session.message = message

    // let session = {}
    // if (!this.conn.session) {
    //   session = this.conn.session = new Session()
    // }
    // this.conn.session.message = message
    // this.conn.session.send = this.conn.dispatch
    cb()
  }

  getStorageData() {
    if (!this.settings.storage) {
      this.settings.storage = new Storage();
    }
    return this.settings.storage;
  }

  saveStorageData() {

  }

  startConversation(session) {
    const _this = this;
    // const dialogs = this.conversation
    let optional_dialog = [];
    (function (dialogs) {
      let dialogCaught
      // const session = this.conn.session
      _this.session = session
      _this.storageCtx = session.storageCtx = session.storageCtx || {}
      session.storageCtx.conversation = session.storageCtx.conversation || dialogs.concat()
      const getMatchedDialog = function (dialogList, index) {
        if (index >= dialogList.length) {
          dialogCaught = dialogList[index]
          execDialog()
          return
        }
        let dialog = dialogList[index]
        let recognizer = dialog.recognizer
        if (recognizer) {
          recognizer = recognizer instanceof Recognizer ? recognizer : new Recognizer(recognizer)
          recognizer.match(session, function (err, isMatched) {
            if (isMatched) {
              dialogCaught = dialog
              execDialog()
            } else {
              getMatchedDialog(dialogs, ++index)
            }
          })
        } else {
          //如果没有定义识别器，则将该dialog加入到待处理队列
          optional_dialog.push(dialog)
          getMatchedDialog(dialogs, ++index)
        }
      }
      getMatchedDialog(dialogs, 0)

      function execDialog() {
        // if (_this.session.isInDialog && !_this.session.interrupt) {
        //   _this.session.dialog(_this.session, _this.session.message)
        // } else {
        //中断时这里还是在执行qna-service
        if (_this.storageCtx && _this.storageCtx.dialogs && _this.storageCtx.dialogs.length) {
          dialogCaught = {
            dialogs: _this.storageCtx.dialogs.concat()
          }
        }
        if (dialogCaught) {
          _this.storageCtx.dialogs = dialogCaught.dialogs.concat()
        } else if (!optional_dialog || !optional_dialog.length) {
          _this.storageCtx.dialogs = _this.rootDialog.dialogs.concat()
        } else {
          _this.storageCtx.dialogs = optional_dialog.shift().dialogs.concat()
        }
        _this.session.dialog = _this.storageCtx.dialogs.shift()
        _this.session.dialog(_this.session, _this.session.message)
        // }
      }
    }.bind({}))(this.conversation)

  }

  add() {
    if (arguments[0] && arguments[0] instanceof Dialog) {
      this.conversation.push(arguments[0])
    } else {
      const dialog = new Dialog(arguments[0], arguments[1])
      this.conversation.push(dialog)
    }
    return this
  }

  triggerAction(args) {
    let dialog = this.conversation.concat().pop()
    dialog.recognizer = args.matches
    return this
  }

  begin(id) {
    let dialog = this.conversation.find((item) => {
      return item.id === id
    })
    this.storageCtx.dialogs = dialog.dialogs
    this.startConversation(this.session)
  }


  /**
   * 
   * @param {*} msg 
   * @param {*} promptsType 
   */
  prompts(msg, promptsType = 'text') {
    // const PROMPTS_TYPES = ['number', 'radio', 'tel', 'check', 'address', 'date', 'time', 'text']
    // if (promptsType && PROMPTS_TYPES.indexOf(promptsType) < 0) {
    //   throw new Error(`unknown prompts type: ${promptsType}`)
    // }

    let message = new Message(promptsType, this.conn.id, msg)
    this.conn.prompts = message

    //TODO::默认所有的prompts都是以text的形式发送给客户端的
    // let prompts = new Prompts('text', this.conn.id, msg, promptsType)
    this.conn.dispatch(msg)
    return this;
  }

  send(message) {
    this.conn.dispatch(message)
  }


}

module.exports = ChatBot