


/**
 * Module dependencies.
 * @private
 */

const Connecter = require('./index')

/**
 * Connector specifically for tytalk
 * Supported message type:
 *  - text
 *  - image
 *  - html
 *  - dialog
 *
 */
class TytalkConnecter extends Connecter {
  constructor() {
    super()
  }

  answerTypeCompiler(answer) {
    let _this = this
    let clientMessage = _this.session.message
    answer = Array.isArray(answer) ? answer : [answer]
    _this.session.answer = answer
    let answers = []
    answer = answer.forEach((item) => {
      let answerType = item.answerType
      switch (answerType) {
        case 'text':
          answers.push(item)
          return answers
        case 'image':
          return item
        case 'html':
          return item
        case 'dialog':
          let dialogNodes = item.content.nodes
          let startNode = dialogNodes.find((node) => {
            return node.isStart
          }) || dialogNodes[0]

          //根据用户当前的选择获取返回给用户的话术和nextNode
          let askLock = false
          getNextNode()
          function getNextNode() {

            let nextNode = _this.session.nextNode || startNode
            if (_this.session.isInDialog) {
              nextNode = _this.session.nextNode
            }
            //1. 返回下一轮的话术
            let currentNode = Object.assign({}, nextNode)
            if (currentNode.dialogType === 'answer') {
              _this.session.isAnswerNode = true
              answers.push({
                answerType: 'text',
                content: currentNode.answer,
                option: {
                  type: currentNode.option.opType,
                  opContent: currentNode.option.opContent
                }
              })
              _this.session.nextNode = dialogNodes.find((node) => {
                return node.dialogId === currentNode.next
              })
              clientMessage = null
              getNextNode()
            } else {
              //记录用户回复信息
              if (_this.session.isInDialog && !_this.session.isAnswerNode) {
                _this.session.dialogData[_this.session.nextNode.name] = clientMessage
              }


              //计算next
              if (currentNode.next && _this.session.dialogData[currentNode.name]) {
                //节点直接有next
                const chinese2num = require('chinese2num');
                function isNumber(string) {
                  return (!(isNaN(string))) || (!isNaN(chinese2num(string)))
                }
                function hasNumber(string) {
                  const s = string.replace(/\s/g, "")
                  let flag = false
                  for (let i = 0; i < s.length; i++) {
                    if (isNumber(s[i])) {
                      flag = true
                      break
                    }
                  }
                  return flag
                }

                if (hasNumber(clientMessage)) {
                  _this.session.back_interrupt = false
                  _this.session.nextNode = dialogNodes.find((node) => {
                    return node.dialogId === currentNode.next
                  })
                } else if (!_this.session.back_interrupt) {
                  _this.session.interrupt = true
                  _this.session.dialog(_this.session, _this.session.message)
                }

              } else if (currentNode.nextFunction) {
                //通过nextFunction计算next节点
                _this.session.nextNode = currentNode.nextFunction(_this.session.dialogData)
              } else if (currentNode.option && currentNode.option.opContent) {
                //根据用户的回复确定next
                //HARD CODE::
                // var reg = new RegExp('^[a-zA-Z]*$')
                // if(clientMessage.test(reg)){

                // }
                let index = clientMessage && clientMessage[0].toLocaleUpperCase().charCodeAt() - 64
                let nextOption = currentNode.option.opContent.find((item) => {
                  //TODO::通过recognizer识别用户是否正常回复了机器人的询问
                  return clientMessage === item.label
                })
                nextOption = nextOption ? nextOption : index <= currentNode.option.opContent.length ? currentNode.option.opContent[index - 1] : null

                if (nextOption) {
                  //找到了用户回复对应的节点
                  clientMessage = nextOption.label
                  _this.session.nextNode = dialogNodes.find((node) => {
                    return node.dialogId === nextOption.next
                  })
                } else {
                  //中断： 多轮对话第一轮
                  if (_this.session.isInDialog && !_this.session.isAnswerNode && !_this.session.back_interrupt) {
                    _this.session.interrupt = true
                    // _this.session.interrupt = false
                    _this.session.dialog(_this.session, _this.session.message)
                  } else {

                    _this.session.back_interrupt = false
                  }
                  //未匹配到选项，说明用户未正常回复(除了刚进入流程时?)
                }
                if (_this.session.nextNode && _this.session.nextNode.dialogType === 'answer') {
                  _this.session.isAnswerNode = true
                  clientMessage = null
                  getNextNode()
                }
              } else {
                //@HARDCODE::最后一个节点的输入不做判断
                _this.session.nextNode = null
              }
              _this.session.isInDialog = true
              if (!askLock && _this.session.nextNode && _this.session.nextNode.dialogType !== 'answer') {
                _this.session.isAnswerNode = false
                askLock = true
                answers.push({
                  answerType: 'text',
                  content: _this.session.nextNode.ask,
                  option: {
                    type: _this.session.nextNode.option.opType,
                    opContent: _this.session.nextNode.option.opContent
                  }
                })
              }
            }
            if (!_this.session.nextNode) {
              _this.session.dialogData[currentNode.name] = clientMessage
              _this.session.isInDialog = false
              let computed = currentNode.computed
              let result = computed ? computed(_this.session.dialogData) : ""
              console.log(' _this.session.dialogData::', _this.session.dialogData)
              _this.session.dialogData = {}
              if (result) {
                return answers.push({
                  answerType: 'text',
                  content: result
                })
              } else {
                return ""
              }
              // let result = computed ? computed(_this.session.dialogData) : _this.session.dialogData
              // _this.session.dialogData = {}
              // return answers.push({
              //   content: `收集到以下信息：${JSON.stringify(result)}`
              // })
            }
            else if (_this.session.nextNode && _this.session.nextNode.dialogType === 'answer') {
              _this.session.isAnswerNode = true
              clientMessage = null
              getNextNode()
            }
          }
          return answers
        default:
          answers.push(item)
          return answers
      }
    })
    return answers
  }

}

module.exports = TytalkConnecter
